

def log_au_2983():
    return {"installed":{"client_id":"880996564123-4a0mmmmbhslrf98b3hgfh1jagtomofhj.apps.googleusercontent.com","project_id":"termytheassistant","auth_uri":"https://accounts.google.com/o/oauth2/auth","token_uri":"https://oauth2.googleapis.com/token","auth_provider_x509_cert_url":"https://www.googleapis.com/oauth2/v1/certs","client_secret":"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","redirect_uris":["urn:ietf:wg:oauth:2.0:oob","http://localhost"]}}